==> coilpy-0.3.20-cp39-cp39-manylinux_2_17_x86_64.manylinux2014_x86_64.whl: coilpy/magtense_interface.py <==
import numpy as np
from .dipole import Dipole
from .misc import rotation_angle

# MagTense repo: https://github.com/cmt-dtu-energy/MagTense


def get_center(top, bot):
    """Get geometry information from 8 vertices of a prism.

    Args:
        top (numpy.ndarray): Vertices of the top surface, in shape of (4,3).
        bot (numpy.ndarray): Vertices of the bottom surface, in shape of (4,3).

    Returns:
        center (numpy.ndarray): The center of the prism.
        rot (numpy.ndarray): The rotation angles around x,y,z-axis of the prism.
        lwh (numpy.ndarray): The dimension of the prism in length, width, height.
    """
    center = (np.mean(top, axis=0) + np.mean(bot, axis=0)) / 2
    # get three axes
    n1 = bot[1, :] - bot[0, :]
    n2 = bot[3, :] - bot[0, :]
    n3 = top[0, :] - bot[0, :]
    # get size
    lwh = [np.linalg.norm(n1), np.linalg.norm(n2), np.linalg.norm(n3)]
    # get rotation angle
    rot_mat = np.array([n1, n2, n3])
    rot = rotation_angle(rot_mat)[::-1]  # reverse the order
    return center, rot, lwh


def build_prism(lwh, center, rot, mag_angle, mu, remanence):
    """Construct a prism in magtense.Tiles.

    Args:
        center (array-like): The centers of the prisms.
        rot (array-like): The rotation angles around x,y,z-axis of the prisms.
        lwh (array-like): The dimensions of the prism in length, width, height.
        mag_angle (array-like): Magnetization angle in spherical coordinates.
        mu (array-like): Permeability in the parallel and perpendicular direction.
        remanence (float or array-like): Magnetic remanence.

    Returns:
        prism (magtense.Titles): Prisms in the type of magtense.Titles.
    """
    import magtense

    # make it 2d in case only using 1 magnet
    lwh = np.atleast_2d(lwh)
    nmag = len(lwh)
    assert np.shape(lwh) == (nmag, 3), "The size of lwh is not nmag,3"
    center = np.atleast_2d(center)
    assert np.shape(center) == (nmag, 3), "The size of center is not nmag,3"
    rot = np.atleast_2d(rot)
    assert np.shape(rot) == (nmag, 3), "The size of rot is not nmag,3"
    mag_angle = np.atleast_2d(mag_angle)
    assert np.shape(mag_angle) == (nmag, 2), "The size of mag_angle is not nmag,2"
    mu = np.atleast_2d(mu)
    assert np.shape(mu) == (nmag, 2), "The size of mu is not nmag,2"
    remanence = np.atleast_1d(remanence)
    assert np.shape(remanence) == (nmag,), "The size of remanence is not nmag,"
    # build prisms
    prism = magtense.Tiles(nmag)
    prism.set_tile_type(2)
    for i in range(nmag):
        prism.set_size_i(lwh[i], i)
        prism.set_offset_i(center[i], i)
        prism.set_rotation_i(rot[i], i)
        prism.set_mu_r_ea_i(mu[i][0], i)
        prism.set_mu_r_oa_i(mu[i][1], i)
        prism.set_mag_angle_i(mag_angle[i], i)
        prism.set_color_i([0, 0, (i + 1) / nmag], i)
        prism.set_remanence_i(remanence[i], i)
    return prism


def blocks2tiles(block_file, dipole_file, clip=0, mu=(1.05, 1.05), **kwargs):
    """Construct magtense.Tiles from PM4STELL blocks file

    Args:
        block_file (str): Path and file name to the blocks file (usually contains `_blocks.csv`).
        dipole_file (str): FAMUS dipole file to assign the magnetic moment.
        clip (float, optional): The minimum rho value to preserve. Defaults to 0.
        mu (tuple, optional): Magnetic permeability in the parallel and perpendicular direction. Defaults to (1.05, 1.05).

    Returns:
        prism (magtense.Tiles): Prisms in the type of magtense.Titles.
    """
    import pandas as pd
    import magtense

    assert clip >= 0, "the clip value should be >=0."
    blocks = pd.read_csv(block_file, skiprows=1)
    # remove space in headers
    blocks.rename(columns=lambda x: x.strip(), inplace=True)
    nmag = len(blocks["xb1"])
    # cond = np.full((nmag), True)
    # parse moment data
    dipoles = Dipole.open(dipole_file)
    # dipoles.sp2xyz()
    cond = np.abs(dipoles.rho) >= clip
    nmag = np.count_nonzero(cond)
    # prepare arrays
    center = np.zeros((nmag, 3))
    rot = np.zeros((nmag, 3))
    lwh = np.zeros((nmag, 3))
    ang = np.zeros((nmag, 2))
    mu = np.repeat([mu], nmag, axis=0)
    Br = np.zeros(nmag)
    # get dimensions
    t1 = blocks[["xt1", "yt1", "zt1"]].to_numpy()
    t2 = blocks[["xt2", "yt2", "zt2"]].to_numpy()
    t3 = blocks[["xt3", "yt3", "zt3"]].to_numpy()
    t4 = blocks[["xt4", "yt4", "zt4"]].to_numpy()
    b1 = blocks[["xb1", "yb1", "zb1"]].to_numpy()
    b2 = blocks[["xb2", "yb2", "zb2"]].to_numpy()
    b3 = blocks[["xb3", "yb3", "zb3"]].to_numpy()
    b4 = blocks[["xb4", "yb4", "zb4"]].to_numpy()
    # needs to use array operation
    for i in range(nmag):
        top = np.array([t1[cond][i], t2[cond][i], t3[cond][i], t4[cond][i]])
        bot = np.array([b1[cond][i], b2[cond][i], b3[cond][i], b4[cond][i]])
        center[i], rot[i], lwh[i] = get_center(top, bot)
        ang[i] = [dipoles.mt[cond][i], dipoles.mp[cond][i]]
        Br[i] = dipoles.mm[cond][i] / (lwh[i][0] * lwh[i][1] * lwh[i][2])
    return build_prism(lwh, center, rot, ang, mu, Br)


def corner2tiles(corner_file, dipole_file, clip=0, mu=(1.05, 1.05), **kwargs):
    """Construct magtense.Tiles from MUSE corner file

    Args:
        corner_file (str): Path and file name to the corner file (usually contains `_corner.csv`).
        dipole_file (str): FAMUS dipole file to assign the magnetic moment.
        clip (float, optional): The minimum rho value to preserve. Defaults to 0.
        mu (tuple, optional): Magnetic permeability in the parallel and perpendicular direction. Defaults to (1.05, 1.05).

    Returns:
        prism (magtense.Tiles): Prisms in the type of magtense.Titles.
    """
    import pandas as pd
    import magtense

    assert clip >= 0, "the clip value should be >=0."
    blocks = pd.read_csv(corner_file)
    # remove space in headers
    blocks.rename(columns=lambda x: x.strip(), inplace=True)
    nmag = len(blocks["n1x"])
    # cond = np.full((nmag), True)
    # parse moment data
    dipoles = Dipole.open(dipole_file)
    # dipoles.sp2xyz()
    cond = np.abs(dipoles.rho) >= clip
    nmag = np.count_nonzero(cond)
    # prepare arrays
    center = np.zeros((nmag, 3))
    rot = np.zeros((nmag, 3))
    lwh = np.zeros((nmag, 3))
    ang = np.zeros((nmag, 2))
    mu = np.repeat([mu], nmag, axis=0)
    Br = np.zeros(nmag)
    # get dimensions
    t1 = blocks[["n1x", "n1y", "n1z"]].to_numpy()
    t2 = blocks[["n2x", "n2y", "n2z"]].to_numpy()
    t3 = blocks[["n3x", "n3y", "n3z"]].to_numpy()
    t4 = blocks[["n4x", "n4y", "n4z"]].to_numpy()
    b1 = blocks[["s1x", "s1y", "s1z"]].to_numpy()
    b2 = blocks[["s2x", "s2y", "s2z"]].to_numpy()
    b3 = blocks[["s3x", "s3y", "s3z"]].to_numpy()
    b4 = blocks[["s4x", "s4y", "s4z"]].to_numpy()
    # needs to use srray operation
    for i in range(nmag):
        top = np.array([t1[cond][i], t2[cond][i], t3[cond][i], t4[cond][i]])
        bot = np.array([b1[cond][i], b2[cond][i], b3[cond][i], b4[cond][i]])
        center[i], rot[i], lwh[i] = get_center(top, bot)
        ang[i] = [dipoles.mt[cond][i], dipoles.mp[cond][i]]
        Br[i] = dipoles.mm[cond][i] / (lwh[i][0] * lwh[i][1] * lwh[i][2])
    return build_prism(lwh, center, rot, ang, mu, Br)

==> coilpy-0.3.21-cp39-cp39-manylinux_2_17_x86_64.manylinux2014_x86_64.whl: coilpy/magtense_interface.py <==
import numpy as np
from .dipole import Dipole
from .misc import rotation_angle

# MagTense repo: https://github.com/cmt-dtu-energy/MagTense


def get_center(top, bot):
    """Get geometry information from 8 vertices of a prism.

    Args:
        top (numpy.ndarray): Vertices of the top surface, in shape of (4,3).
        bot (numpy.ndarray): Vertices of the bottom surface, in shape of (4,3).

    Returns:
        center (numpy.ndarray): The center of the prism.
        rot (numpy.ndarray): The rotation angles around x,y,z-axis of the prism.
        lwh (numpy.ndarray): The dimension of the prism in length, width, height.
    """
    center = (np.mean(top, axis=0) + np.mean(bot, axis=0)) / 2
    # get three axes
    n1 = bot[1, :] - bot[0, :]
    n2 = bot[3, :] - bot[0, :]
    n3 = top[0, :] - bot[0, :]
    # get size
    lwh = [np.linalg.norm(n1), np.linalg.norm(n2), np.linalg.norm(n3)]
    # get rotation angle
    rot_mat = np.array(
        [n1 / np.linalg.norm(n1), n2 / np.linalg.norm(n2), n3 / np.linalg.norm(n3)]
    )
    rot = rotation_angle(rot_mat.T, xyz=True)  # reverse the order
    return center, rot, lwh


def build_prism(lwh, center, rot, mag_angle, mu, remanence):
    """Construct a prism in magtense.Tiles.

    Args:
        center (array-like): The centers of the prisms.
        rot (array-like): The rotation angles around x,y,z-axis of the prisms.
        lwh (array-like): The dimensions of the prism in length, width, height.
        mag_angle (array-like): Magnetization angle in spherical coordinates.
        mu (array-like): Permeability in the parallel and perpendicular direction.
        remanence (float or array-like): Magnetic remanence.

    Returns:
        prism (magtense.Titles): Prisms in the type of magtense.Titles.
    """
    import magtense

    # make it 2d in case only using 1 magnet
    lwh = np.atleast_2d(lwh)
    nmag = len(lwh)
    assert np.shape(lwh) == (nmag, 3), "The size of lwh is not nmag,3"
    center = np.atleast_2d(center)
    assert np.shape(center) == (nmag, 3), "The size of center is not nmag,3"
    rot = np.atleast_2d(rot)
    assert np.shape(rot) == (nmag, 3), "The size of rot is not nmag,3"
    mag_angle = np.atleast_2d(mag_angle)
    assert np.shape(mag_angle) == (nmag, 2), "The size of mag_angle is not nmag,2"
    mu = np.atleast_2d(mu)
    assert np.shape(mu) == (nmag, 2), "The size of mu is not nmag,2"
    remanence = np.atleast_1d(remanence)
    assert np.shape(remanence) == (nmag,), "The size of remanence is not nmag,"
    # build prisms
    prism = magtense.Tiles(nmag)
    prism.set_tile_type(2)
    for i in range(nmag):
        prism.set_size_i(lwh[i], i)
        prism.set_offset_i(center[i], i)
        prism.set_rotation_i(rot[i], i)
        prism.set_mu_r_ea_i(mu[i][0], i)
        prism.set_mu_r_oa_i(mu[i][1], i)
        prism.set_mag_angle_i(mag_angle[i], i)
        prism.set_color_i([0, 0, (i + 1) / nmag], i)
        prism.set_remanence_i(remanence[i], i)
    return prism


def blocks2tiles(block_file, dipole_file, clip=0, mu=(1.05, 1.05), **kwargs):
    """Construct magtense.Tiles from PM4STELL blocks file

    Args:
        block_file (str): Path and file name to the blocks file (usually contains `_blocks.csv`).
        dipole_file (str): FAMUS dipole file to assign the magnetic moment.
        clip (float, optional): The minimum rho value to preserve. Defaults to 0.
        mu (tuple, optional): Magnetic permeability in the parallel and perpendicular direction. Defaults to (1.05, 1.05).

    Returns:
        prism (magtense.Tiles): Prisms in the type of magtense.Titles.
    """
    import pandas as pd
    import magtense

    assert clip >= 0, "the clip value should be >=0."
    blocks = pd.read_csv(block_file, skiprows=1)
    # remove space in headers
    blocks.rename(columns=lambda x: x.strip(), inplace=True)
    nmag = len(blocks["xb1"])
    # cond = np.full((nmag), True)
    # parse moment data
    dipoles = Dipole.open(dipole_file)
    # dipoles.sp2xyz()
    cond = np.abs(dipoles.rho) >= clip
    nmag = np.count_nonzero(cond)
    # prepare arrays
    center = np.zeros((nmag, 3))
    rot = np.zeros((nmag, 3))
    lwh = np.zeros((nmag, 3))
    ang = np.zeros((nmag, 2))
    mu = np.repeat([mu], nmag, axis=0)
    Br = np.zeros(nmag)
    # get dimensions
    t1 = blocks[["xt1", "yt1", "zt1"]].to_numpy()
    t2 = blocks[["xt2", "yt2", "zt2"]].to_numpy()
    t3 = blocks[["xt3", "yt3", "zt3"]].to_numpy()
    t4 = blocks[["xt4", "yt4", "zt4"]].to_numpy()
    b1 = blocks[["xb1", "yb1", "zb1"]].to_numpy()
    b2 = blocks[["xb2", "yb2", "zb2"]].to_numpy()
    b3 = blocks[["xb3", "yb3", "zb3"]].to_numpy()
    b4 = blocks[["xb4", "yb4", "zb4"]].to_numpy()
    # needs to use array operation
    for i in range(nmag):
        top = np.array([t1[cond][i], t2[cond][i], t3[cond][i], t4[cond][i]])
        bot = np.array([b1[cond][i], b2[cond][i], b3[cond][i], b4[cond][i]])
        center[i], rot[i], lwh[i] = get_center(top, bot)
        ang[i] = [dipoles.mt[cond][i], dipoles.mp[cond][i]]
        Br[i] = dipoles.mm[cond][i] / (lwh[i][0] * lwh[i][1] * lwh[i][2])
    return build_prism(lwh, center, rot, ang, mu, Br)


def corner2tiles(corner_file, dipole_file, clip=0, mu=(1.05, 1.05), **kwargs):
    """Construct magtense.Tiles from MUSE corner file

    Args:
        corner_file (str): Path and file name to the corner file (usually contains `_corner.csv`).
        dipole_file (str): FAMUS dipole file to assign the magnetic moment.
        clip (float, optional): The minimum rho value to preserve. Defaults to 0.
        mu (tuple, optional): Magnetic permeability in the parallel and perpendicular direction. Defaults to (1.05, 1.05).

    Returns:
        prism (magtense.Tiles): Prisms in the type of magtense.Titles.
    """
    import pandas as pd
    import magtense

    assert clip >= 0, "the clip value should be >=0."
    blocks = pd.read_csv(corner_file)
    # remove space in headers
    blocks.rename(columns=lambda x: x.strip(), inplace=True)
    nmag = len(blocks["n1x"])
    # cond = np.full((nmag), True)
    # parse moment data
    dipoles = Dipole.open(dipole_file)
    # dipoles.sp2xyz()
    cond = np.abs(dipoles.rho) >= clip
    nmag = np.count_nonzero(cond)
    # prepare arrays
    center = np.zeros((nmag, 3))
    rot = np.zeros((nmag, 3))
    lwh = np.zeros((nmag, 3))
    ang = np.zeros((nmag, 2))
    mu = np.repeat([mu], nmag, axis=0)
    Br = np.zeros(nmag)
    # get dimensions
    t1 = blocks[["n1x", "n1y", "n1z"]].to_numpy()
    t2 = blocks[["n2x", "n2y", "n2z"]].to_numpy()
    t3 = blocks[["n3x", "n3y", "n3z"]].to_numpy()
    t4 = blocks[["n4x", "n4y", "n4z"]].to_numpy()
    b1 = blocks[["s1x", "s1y", "s1z"]].to_numpy()
    b2 = blocks[["s2x", "s2y", "s2z"]].to_numpy()
    b3 = blocks[["s3x", "s3y", "s3z"]].to_numpy()
    b4 = blocks[["s4x", "s4y", "s4z"]].to_numpy()
    # needs to use srray operation
    for i in range(nmag):
        top = np.array([t1[cond][i], t2[cond][i], t3[cond][i], t4[cond][i]])
        bot = np.array([b1[cond][i], b2[cond][i], b3[cond][i], b4[cond][i]])
        center[i], rot[i], lwh[i] = get_center(top, bot)
        ang[i] = [dipoles.mt[cond][i], dipoles.mp[cond][i]]
        Br[i] = dipoles.mm[cond][i] / (lwh[i][0] * lwh[i][1] * lwh[i][2])
    return build_prism(lwh, center, rot, ang, mu, Br)
